;-----------------------------------------------------------------------------
;                                   SBC-2051
;                             System Check Firmware
;                                 Version 1.1
;                         Copyright (c) 1996,1997 E.D.A.
;-----------------------------------------------------------------------------
;
$MOD51
$TITLE(SBC-2051 SYSTEM CHECK FIRMWARE)
$PAGEWIDTH(132)
$DEBUG
$OBJECT
$NOPAGING

;
;-----------------------------------------------------------------------------
;                                 Equates
;-----------------------------------------------------------------------------
;
NULL		equ		0		;null
CR			equ		13		;carriage return
LF			equ		10		;line feed
BS			equ		127		;back space

BAUD_1200	equ		0
BAUD_2400   equ		1
BAUD_4800   equ		2
BAUD_9600   equ		3
BAUD_19200  equ		4

REC_BUFFER_SIZE	equ	10
STACK_SIZE	equ		50

;
;-----------------------------------------------------------------------------
;                         Byte Variable Declarations
;-----------------------------------------------------------------------------
;
			DSEG	AT	030h		;skip over register banks
									;and bit addressable RAM
rec_buf:	DS		REC_BUFFER_SIZE
end_buf:	DS		1
rlen:		DS		1
rptr:		DS		1
rec_addr: 	DS		1
intcnt:		DS		1
mstimer:	DS		1
butcnt:		DS		1
;add new byte variables here

stack:		DS		STACK_SIZE		;allocate space for the stack

;
;-----------------------------------------------------------------------------
;                         Bit Variable Declarations
;-----------------------------------------------------------------------------
;
			BSEG	AT	000h
message:	DBIT	1
butflag:	DBIT	1
;add new bit variables here

;
;-----------------------------------------------------------------------------
;                            Hardware Definitions
;-----------------------------------------------------------------------------
;
LED			BIT		P1.0	;LED control output
BUTTON		BIT		P1.1	;pushbutton input
SW1			BIT		P3.2	;DIP switch 1 input
SW2			BIT		P3.3	;DIP switch 2 input
SW3			BIT		P3.4	;DIP switch 3 input
SW4			BIT		P3.5	;DIP switch 4 input
WATCHDOG	BIT		P3.7	;DS1232 strobe

;
;-----------------------------------------------------------------------------
;                             Interrupt Vectors
;-----------------------------------------------------------------------------
;
		CSEG	AT	000h
		ORG	000h
		ljmp main			;RESET

		ORG 003h
		ljmp dummy			;IE0 


		ORG 00Bh			;TF0
		ljmp t0isr

		ORG 013h			;IE1
		ljmp dummy

		ORG 01Bh			;TF1
		ljmp dummy

		ORG 023h			;RI + TI
		ljmp dummy

dummy:	reti

;
;-----------------------------------------------------------------------------
;                                 strings
;-----------------------------------------------------------------------------
;
		ORG	100H

greeting:
		DB	CR,LF,'  -------------------------------------- '
		DB	CR,LF,' | Model SBC-2051 Single Board Computer |'
		DB	CR,LF,' |   System Check Firmware Version 1.1  |'
		DB	CR,LF,' |    Copyright (c) 1996,1997 E.D.A.    |'
		DB	CR,LF,'  -------------------------------------- '
		DB	CR,LF,CR,LF,' Type ? and press ENTER for help.',CR,LF,CR,LF,'>',NULL
		;note - strings must be null terminated

buttonstring:
		DB	CR,LF,'Button Pressed',CR,LF,CR,LF,'>',NULL

OKstring:
		DB	CR,LF,'OK',NULL

help:	
		DB	CR,LF,'HELP:'
		DB	CR,LF,'I - Information'
		DB	CR,LF,'P - Port1 Status'
		DB	CR,LF,'S - DIP Switch Status',NULL

info:
		DB	CR,LF,'Model: SBC-2051'
		DB	CR,LF,'Date:  1/13/97'
		DB	CR,LF,'Ver:   1.1',NULL

port1_string:
		DB	CR,LF,'PORT1=0x',NULL

hex:
		DB	'0123456789ABCDEF',NULL

bad_cmd:
		DB	CR,LF,'Bad command',NULL

prompt:
		DB	CR,LF,CR,LF,'>',NULL

SW1string:
		DB	CR,LF,'SW1 = ',NULL
SW2string:
		DB	CR,LF,'SW2 = ',NULL
SW3string:
		DB	CR,LF,'SW3 = ',NULL
SW4string:
		DB	CR,LF,'SW4 = ',NULL

ONstring:	
		DB	'ON',NULL

OFFstring:
		DB	'OFF',NULL

;
;-----------------------------------------------------------------------------
;                             Initialization
;-----------------------------------------------------------------------------
;
main:	mov	sp,#stack		;initialize stack pointer
		using 0				;select register bank0

		mov a,#002h
		mov TMOD,a			;timer 0 is 8-bit auto reload

		mov a,#26
		mov TH0,a			;set timer 0 to interrupt every 250us (assumes 11.0592Mhz crystal)
		mov TL0,a

		setb TR0			;start timer 0

		setb ET0			;enable timer 0 interrupt

		mov intcnt,#40		;initialize interrupt counter
		mov mstimer,#250	;initialize millisecond timer

		mov	a,#BAUD_19200
		call init_comm		;initialize serial communications

		setb EA 			;enable interrupts

		mov DPTR,#greeting	;print opening greeting
		call printstring

;
;-----------------------------------------------------------------------------
;                              main program loop
;-----------------------------------------------------------------------------
;
loop:	setb WATCHDOG

		call keyboard		;monitor user input from keyboard

		jnb message,loop01	;process user message when message flag is set
		call proc_mess

loop01:
		jnb butflag,loop02	;print string when button flag set
		clr butflag
		mov dptr,#buttonstring
		call printstring

loop02:
		ljmp loop

;
;-----------------------------------------------------------------------------
;                      timer 0 interrupt service routine
;
; This interrupt service routine is executed once every 250us in response to
; a timer0 overflow.  Counters are used to make a 10ms timer and a 250ms 
; timer.  The button is also debounced here.
;-----------------------------------------------------------------------------
;
t0isr:	push ACC			;save the processor state on the stack
		push B
		push DPH
		push DPL
		push PSW

		using 1				;select register bank 1

		clr WATCHDOG

		djnz intcnt,t0_01	;it takes 40 250us interrupts to make 10ms
		mov intcnt,#40

		djnz mstimer,t0_01
		mov mstimer,#50
		cpl LED				;toggle LED every 500ms

t0_01:
		jb BUTTON,t0_02		;set button flag if button depressed for 200 consecutive interrupts (50ms)
		mov a,#200
		clr c
		subb a,butcnt
		jc t0_03
		jz t0_03
		inc butcnt
		mov a,butcnt
		cjne a,#200,t0_03
		setb butflag
		jmp	t0_03

t0_02:
		clr a
		mov butcnt,a

t0_03:
		pop PSW				;restore the processor state from the stack
		pop DPL
		pop DPH
		pop B
		pop ACC
		reti

;
;-----------------------------------------------------------------------------
;                                 keyboard
;
; This function checks for received characters and puts them in the receive
; buffer.  A character is deleted from the receive buffer when a back space
; character is received.  The message flag is set when a carriage return is
; received.
;-----------------------------------------------------------------------------
;
keyboard:
		call getchar
		jz kb_end				;if no byte received then exit

		mov R0,rptr				;get receive buffer pointer
		
		jb message,kb_break

		mov @R0,a				;put received byte in buffer

		cjne a,#CR,kb_case1		;check for carriage return
		setb message
		mov a,#end_buf
		clr c
		subb a,rptr
		jc kb_01
		jz kb_01
		inc rptr
kb_01:
		mov R0,rptr
		mov @R0,#NULL
		jmp kb_break

kb_case1:	
		cjne a,#BS,kb_default	;check for back space
		mov a,rlen
		jz kb_break
		dec rlen
		dec rptr
		mov a,#BS
		call putchar
		jmp kb_break

kb_default:
		mov a,#end_buf			;if buffer not full then increment pointer & echo received byte
		clr c
		subb a,rptr
		jc kb_break
		jz kb_break
		inc rptr
		inc rlen
		mov a,@R0
		call putchar

kb_break:

kb_end:
		ret


;
;-----------------------------------------------------------------------------
;                                  proc_mess
;
; This function examines the contents of the receive buffer and processes the
; user message.  Messages consist of single letter commands followed by the
; ENTER key.
;-----------------------------------------------------------------------------
;
proc_mess:
		mov a,rlen
		jnz pm_01

		mov dptr,#OKstring			;respond with OK is user types ENTER alone (rlen=0)
		call printstring
		jmp end_pm

pm_01:
		mov a,rec_buf

		cjne a,#'?',pm_case1		;display help screen if user types ? and ENTER
		mov dptr,#help
		call printstring
		jmp pm_default

pm_case1:
		cjne a,#'I',pm_case2		;display information screen if user types I and ENTER
		mov dptr,#info
		call printstring
		jmp pm_break

pm_case2:
		cjne a,#'P',pm_case3		;display status of Port1 if user types P and ENTER
		mov dptr,#port1_string
		call printstring
		mov dptr,#hex
		mov a,P1
		anl a,#0F0h
		swap a
		movc a,@a+dptr
		call putchar
		mov a,P1
		anl a,#00Fh
		movc a,@a+dptr
		call putchar
		jmp pm_break

pm_case3:
		cjne a,#'S',pm_default		;display status of DIP switches if user types S and ENTER
		mov dptr,#SW1string
		call printstring
		mov c,SW1
		call printswitch

		mov dptr,#SW2string
		call printstring
		mov c,SW2
		call printswitch

		mov dptr,#SW3string
		call printstring
		mov c,SW3
		call printswitch

		mov dptr,#SW4string
		call printstring
		mov c,SW4
		call printswitch
		jmp pm_break

pm_default:
		mov dptr,#bad_cmd			;display bad command message
		call printstring

pm_break:
end_pm:
		mov dptr,#prompt
		call printstring
		clr a
		mov rlen,a
		mov rptr,#rec_buf
		clr message
		ret

;
; prints 'ON' if carry clear, 'OFF' if carry set
;
printswitch:
		mov dptr,#OFFstring
		jc ps_01
		mov dptr,#ONstring
ps_01:	call printstring
		ret



;
;-----------------------------------------------------------------------------
;                                  init_comm
;
; This function intializes the on-chip uart.  The desired baud rate is passed
; in the a accumulator.  This function assumes the use of an 11.0592Mhz
; crystal.
;-----------------------------------------------------------------------------
;
init_comm:
		cjne a,#BAUD_19200,ic_case1
		mov a,#080h			;set double baud rate bit
		orl PCON,a
		mov TH1,#0FDh
		mov TL1,#0FDh
		jmp ic_break

ic_case1: 
		cjne a,#BAUD_9600,ic_case2
		mov a,#07Fh			;clear double baud rate bit
		anl PCON,a
		mov TH1,#0FDh
		mov TL1,#0FDh
		jmp ic_break

ic_case2:	
		cjne a,#BAUD_4800,ic_case3
		mov a,#07Fh			;clear double baud rate bit
		anl PCON,a
		mov TH1,#0FAh
		mov TL1,#0FAh
		jmp ic_break

ic_case3:	
		cjne a,#BAUD_2400,ic_default
		mov a,#07Fh			;clear double baud rate bit
		anl PCON,a
		mov TH1,#0F4h
		mov TL1,#0F4h
		jmp ic_break

ic_default:					;default to 1200 baud
		mov a,#07Fh			;clear double baud rate bit
		anl PCON,a
		mov TH1,#0E8h
		mov TL1,#0E8h

ic_break:
		clr	a
		mov rlen,a			;clear receive message length counter
		mov rptr,#rec_buf	;clear receive message buffer pointer
		clr message			;clear message flag

		anl TMOD,#00Fh		;setup timer1 as 8-bit auto reload timer
		orl TMOD,#020h
		mov SCON,#040h		;set uart to 8-bit variable baud rate
        mov a,SBUF			;clear any pending serial interrupt
		clr RI				;clear receive interrupt flag
		setb TI				;clear transmitter interrupt flag
		setb TR1			;start timer 1
		setb REN			;enable serial reception
		clr message			;clear message flag
		ret

;
;-----------------------------------------------------------------------------
;                                   putchar
;
; This function writes a character to the standard output (uart in this case).
;-----------------------------------------------------------------------------
;
putchar:  
		jnb TI,putchar
		clr TI
		mov SBUF,a
		ret

;
;-----------------------------------------------------------------------------
;                                   getchar
;
; This function reads a character from the standard input (uart in this case).
; Unlike the standard c library function, this getchar returns a NULL if no
; character has been received.  Standard c library getchar functions wait in
; a loop until a character is received.  Waiting is not a desirable
; characteristic in an embedded system.
;-----------------------------------------------------------------------------
;
getchar:
		jnb RI,no_char
		mov a,SBUF
		clr RI
		ret
no_char:
		clr a		;return NULL (no character received)
		ret

;
;-----------------------------------------------------------------------------
;                                  printstring
;
; This function writes a string to the standard output using the putchar
; function.  The address of the string is passed in the DPTR.  The string 
; must be NULL terminated.  This function can only be used to print strings 
; stored in program memory space.
;-----------------------------------------------------------------------------
;
printstring:
		clr a
		movc a,@a+dptr		;get character
		jz end_print		;exit if NULL character detected
		call putchar		;print character
		inc dptr			;index to next character
		jmp	printstring
end_print:
		ret

	END
